@@ -1,5 +1,5 @@
 package Ryu::Source;
-$Ryu::Source::VERSION = '0.006';
+$Ryu::Source::VERSION = '0.007';
 use strict;
 use warnings;
 
@@ -64,33 +64,6 @@ sub new {
     $self->SUPER::new(%args);
 }
 
-=head2 chained
-
-Returns a new L<Ryu::Source> chained from this one.
-
-=cut
-
-sub chained {
-    use Scalar::Util qw(weaken);
-    use namespace::clean qw(weaken);
-
-    my ($self) = shift;
-    if(my $class = ref($self)) {
-        my $src = $class->new(
-            new_future => $self->{new_future},
-            parent     => $self,
-            @_
-        );
-        weaken($src->{parent});
-        push @{$self->{children}}, $src;
-        $log->tracef("Constructing chained source for %s from %s (%s)", $src->label, $self->label, $future_state->($self->completed));
-        return $src;
-    } else {
-        my $src = $self->new(@_);
-        $log->tracef("Constructing chained source for %s with no parent", $src->label, $self->label);
-    }
-}
-
 =head2 describe
 
 Returns a string describing this source and any parents - typically this will result in a chain
@@ -225,6 +198,8 @@ sub is_paused { $_[0]->{is_paused} }
 
 Not yet implemented.
 
+Requires timing support, see implementations such as L<Ryu::Async> instead.
+
 =cut
 
 sub debounce {
@@ -268,6 +243,12 @@ sub map : method {
     $self->each_while_source(sub { $src->emit($_->$code) }, $src);
 }
 
+=head2 split
+
+Splits the input into chunks. By default, will split into characters.
+
+=cut
+
 sub split : method {
     my ($self, $delim) = @_;
     $delim //= qr//;
@@ -280,6 +261,15 @@ sub split : method {
     $self->each_while_source(sub { $src->emit($_) for split $delim, $_ }, $src);
 }
 
+=head2 chunksize
+
+Splits input into fixed-size chunks.
+
+Note that output is always guaranteed to be a full chunk - if there is partial input
+at the time the input stream finishes, those extra bytes will be discarded.
+
+=cut
+
 sub chunksize : method {
     my ($self, $size) = @_;
     die 'need positive chunk size parameter' unless $size && $size > 0;
@@ -342,11 +332,19 @@ sub combine_latest : method {
     retain_future(
         Future->needs_any(
             map $_->completed, @sources
-        )->on_ready($combined->completed)
+        )->on_ready(sub {
+            @value = ();
+            return if $combined->completed->is_ready;
+            shift->on_ready($combined->completed)
+        })
     );
     $combined
 }
 
+=head2 with_latest_from
+
+=cut
+
 sub with_latest_from : method {
     use Scalar::Util qw(blessed);
     use namespace::clean qw(blessed);
@@ -370,6 +368,7 @@ sub with_latest_from : method {
     });
     $self->completed->on_ready($combined->completed);
     $self->completed->on_ready(sub {
+        @value = ();
         return if $combined->is_ready;
         shift->on_ready($combined->completed);
     });
@@ -401,6 +400,15 @@ sub merge : method {
     $combined
 }
 
+=head2 apply
+
+Used for setting up multiple streams.
+
+Accepts a variable number of coderefs, will call each one and gather L<Ryu::Source>
+results.
+
+=cut
+
 sub apply : method {
     use Variable::Disposition qw(retain_future);
     use namespace::clean qw(retain_future);
@@ -424,6 +432,10 @@ sub apply : method {
 
 =head2 distinct
 
+Emits new distinct items.
+
+Given 1,2,3,2,3,2,4,1,5, you'd get the sequence 1,2,3,4,5.
+
 =cut
 
 sub distinct {
@@ -455,6 +467,8 @@ sub distinct {
 
 =head2 skip
 
+Skips the first N items.
+
 =cut
 
 sub skip {
@@ -474,6 +488,8 @@ sub skip {
 
 =head2 skip_last
 
+Skips the last N items.
+
 =cut
 
 sub skip_last {
@@ -495,6 +511,11 @@ sub skip_last {
 
 =head2 take
 
+Takes a limited number of items.
+
+Given a sequence of C< 1,2,3,4,5 > and C<< ->take(3) >>, you'd get 1,2,3 and then the stream
+would finish.
+
 =cut
 
 sub take {
@@ -517,19 +538,6 @@ sub take {
     }, $src);
 }
 
-sub each_while_source {
-    use Scalar::Util qw(refaddr);
-    use List::UtilsBy qw(extract_by);
-    use namespace::clean qw(refaddr extract_by);
-    my ($self, $code, $src) = @_;
-    $self->each($code);
-    $src->completed->on_ready(sub {
-        my $count = extract_by { refaddr($_) == refaddr($code) } @{$self->{on_item}};
-        $log->tracef("->e_w_s completed on %s for refaddr 0x%x", $self->describe, refaddr($self));
-    });
-    $src
-}
-
 =head2 some
 
 =cut
@@ -994,10 +1002,58 @@ sub await {
     $self
 }
 
+=head2 finish
+
+Mark this source as completed.
+
+=cut
+
 sub finish { shift->completed->done }
 
 sub refresh { }
 
+=head1 METHODS - Internal
+
+=head2 chained
+
+Returns a new L<Ryu::Source> chained from this one.
+
+=cut
+
+sub chained {
+    use Scalar::Util qw(weaken);
+    use namespace::clean qw(weaken);
+
+    my ($self) = shift;
+    if(my $class = ref($self)) {
+        my $src = $class->new(
+            new_future => $self->{new_future},
+            parent     => $self,
+            @_
+        );
+        weaken($src->{parent});
+        push @{$self->{children}}, $src;
+        $log->tracef("Constructing chained source for %s from %s (%s)", $src->label, $self->label, $future_state->($self->completed));
+        return $src;
+    } else {
+        my $src = $self->new(@_);
+        $log->tracef("Constructing chained source for %s with no parent", $src->label, $self->label);
+    }
+}
+
+sub each_while_source {
+    use Scalar::Util qw(refaddr);
+    use List::UtilsBy qw(extract_by);
+    use namespace::clean qw(refaddr extract_by);
+    my ($self, $code, $src) = @_;
+    $self->each($code);
+    $src->completed->on_ready(sub {
+        my $count = extract_by { refaddr($_) == refaddr($code) } @{$self->{on_item}};
+        $log->tracef("->e_w_s completed on %s for refaddr 0x%x", $self->describe, refaddr($self));
+    });
+    $src
+}
+
 sub DESTROY {
     my ($self) = @_;
     return if ${^GLOBAL_PHASE} eq 'DESTRUCT';
